import * as tslib_1 from "tslib";
import { ContentTypes, FetchHookTypes } from '../types';
import { ModulePrefix } from '../index';
import { ContentTypeToString, StringToContentType } from '../util/Filters';
import Meta from '../plugin/inheritable/Meta';
var orNull = function (value) {
    return value ? value : null;
};
var LoaderRequestToFetches = function (slug, store) {
    var requests = [];
    for (var _i = 0, slug_1 = slug; _i < slug_1.length; _i++) {
        var requestElement = slug_1[_i];
        if (typeof requestElement === 'string') {
            requests.push(store.dispatch(ModulePrefix + "_page/load", {
                slug: requestElement,
                type: ContentTypes.Page
            }));
        }
        else {
            var _a = ConsumeObject(requestElement), slug_2 = _a.slug, contentType = _a.contentType;
            var type = ContentTypeToString(contentType);
            requests.push(store.dispatch(ModulePrefix + "_" + type + "/load", {
                slug: slug_2,
                type: StringToContentType(type)
            }));
        }
    }
    return requests;
};
var ConsumeObject = function (obj) {
    var slug = obj.slug;
    var dataName = slug;
    var metaSource = null;
    if ('dataName' in obj) {
        dataName = obj.dataName;
    }
    if (('meta' in obj && obj.meta === true)) {
        metaSource = dataName;
    }
    var contentType = 'post' in obj && obj.post === true
        ? ContentTypes.Post
        : ContentTypes.Page;
    return {
        slug: slug,
        dataName: dataName,
        meta: metaSource,
        contentType: contentType
    };
};
export default (function (loaderRequest, createdOrAsync) {
    var _a, _b;
    if (createdOrAsync === void 0) { createdOrAsync = FetchHookTypes.Created; }
    var metaSource;
    if (!loaderRequest || (Array.isArray(loaderRequest) && loaderRequest.length < 1)) {
        throw new Error('Slug no provided');
    }
    var mixin = {};
    var computed = {};
    if (createdOrAsync === FetchHookTypes.Created) {
        // Appears in Vue version
        var requests_1 = [];
        var type_1 = ContentTypeToString(ContentTypes.Page);
        if (typeof loaderRequest === 'string') {
            metaSource = loaderRequest;
            computed = tslib_1.__assign({}, computed, (_a = {}, _a[loaderRequest] = function () {
                return orNull(this.$store.state[ModulePrefix + "_" + type_1][type_1][loaderRequest]);
            }, _a));
            mixin = tslib_1.__assign({}, mixin, { created: function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function () {
                        return tslib_1.__generator(this, function (_a) {
                            switch (_a.label) {
                                case 0: return [4 /*yield*/, this.$store.dispatch(ModulePrefix + "_page/load", {
                                        slug: loaderRequest,
                                        type: ContentTypes.Page
                                    })];
                                case 1:
                                    _a.sent();
                                    return [2 /*return*/];
                            }
                        });
                    });
                } });
        }
        else if (Array.isArray(loaderRequest)) {
            var _loop_1 = function (requestElement) {
                var _a, _b;
                if (typeof requestElement === 'string') {
                    if (!metaSource) {
                        metaSource = requestElement;
                    }
                    computed = tslib_1.__assign({}, computed, (_a = {}, _a[requestElement] = function () {
                        return orNull(this.$store.state[ModulePrefix + "_page"].page[requestElement]);
                    }, _a));
                    requests_1.push({
                        slug: requestElement,
                        type: ContentTypes.Page
                    });
                }
                else {
                    var _c = ConsumeObject(requestElement), slug_3 = _c.slug, dataName_1 = _c.dataName, meta = _c.meta, contentType = _c.contentType;
                    var type_2 = ContentTypeToString(contentType);
                    if (meta !== null) {
                        metaSource = meta;
                    }
                    computed = tslib_1.__assign({}, computed, (_b = {}, _b[dataName_1] = function () {
                        return orNull(this.$store.state[ModulePrefix + "_" + type_2][type_2][slug_3]);
                    }, _b));
                    requests_1.push({
                        slug: slug_3,
                        type: contentType
                    });
                }
            };
            // loaderRequest - array of request elements in this example
            for (var _i = 0, loaderRequest_1 = loaderRequest; _i < loaderRequest_1.length; _i++) {
                var requestElement = loaderRequest_1[_i];
                _loop_1(requestElement);
            }
            mixin = tslib_1.__assign({}, mixin, { created: function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function () {
                        var _this = this;
                        return tslib_1.__generator(this, function (_a) {
                            switch (_a.label) {
                                case 0: return [4 /*yield*/, Promise.all(requests_1.map(function (request) {
                                        var type = ContentTypeToString(request.type);
                                        return _this.$store.dispatch(ModulePrefix + "_" + type + "/load", request);
                                    }))];
                                case 1:
                                    _a.sent();
                                    return [2 /*return*/];
                            }
                        });
                    });
                } });
        }
        else {
            var _c = ConsumeObject(loaderRequest), slug_4 = _c.slug, dataName_2 = _c.dataName, meta = _c.meta, contentType = _c.contentType;
            var type_3 = ContentTypeToString(contentType);
            if (meta !== null) {
                metaSource = meta;
            }
            computed = tslib_1.__assign({}, computed, (_b = {}, _b[dataName_2] = function () {
                return orNull(this.$store.state[ModulePrefix + "_" + type_3][type_3][slug_4]);
            }, _b));
            mixin = tslib_1.__assign({}, mixin, { created: function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function () {
                        return tslib_1.__generator(this, function (_a) {
                            switch (_a.label) {
                                case 0: return [4 /*yield*/, this.$store.dispatch(ModulePrefix + "_" + type_3 + "/load", {
                                        slug: slug_4,
                                        type: ContentTypes.Page
                                    })];
                                case 1:
                                    _a.sent();
                                    return [2 /*return*/];
                            }
                        });
                    });
                } });
        }
        mixin = tslib_1.__assign({}, mixin, { computed: computed });
    }
    else if (createdOrAsync === FetchHookTypes.AsyncData) {
        // Appears in Nuxt version
        var requests = [];
        if (typeof loaderRequest === 'string') {
            metaSource = loaderRequest;
            mixin = tslib_1.__assign({}, mixin, { asyncData: function (_a) {
                    var store = _a.store;
                    return tslib_1.__awaiter(this, void 0, void 0, function () {
                        var _b, typeAsString;
                        return tslib_1.__generator(this, function (_c) {
                            switch (_c.label) {
                                case 0:
                                    typeAsString = ContentTypeToString(ContentTypes.Page);
                                    return [4 /*yield*/, store.dispatch(ModulePrefix + "_page/load", {
                                            slug: loaderRequest,
                                            type: ContentTypes.Page
                                        })];
                                case 1:
                                    _c.sent();
                                    return [2 /*return*/, (_b = {},
                                            _b[loaderRequest] = orNull(store.state[ModulePrefix + "_" + typeAsString][typeAsString][loaderRequest]),
                                            _b)];
                            }
                        });
                    });
                } });
        }
        else if (Array.isArray(loaderRequest)) {
            // slug - array of request elements in this example
            mixin = tslib_1.__assign({}, mixin, { asyncData: function (_a) {
                    var store = _a.store;
                    return tslib_1.__awaiter(this, void 0, void 0, function () {
                        var _b, _c, data, _i, loaderRequest_2, requestElement, _d, slug, dataName, meta, contentType, type;
                        return tslib_1.__generator(this, function (_e) {
                            switch (_e.label) {
                                case 0: return [4 /*yield*/, Promise.all(LoaderRequestToFetches(loaderRequest, store))];
                                case 1:
                                    _e.sent();
                                    data = {};
                                    for (_i = 0, loaderRequest_2 = loaderRequest; _i < loaderRequest_2.length; _i++) {
                                        requestElement = loaderRequest_2[_i];
                                        if (typeof requestElement === 'string') {
                                            if (!metaSource) {
                                                metaSource = requestElement;
                                            }
                                            data = tslib_1.__assign({}, data, (_b = {}, _b[requestElement] = orNull(store.state[ModulePrefix + "_page"].page[requestElement]), _b));
                                        }
                                        else {
                                            _d = ConsumeObject(requestElement), slug = _d.slug, dataName = _d.dataName, meta = _d.meta, contentType = _d.contentType;
                                            type = ContentTypeToString(contentType);
                                            if (meta !== null) {
                                                metaSource = meta;
                                            }
                                            data = tslib_1.__assign({}, data, (_c = {}, _c[dataName] = orNull(store.state[ModulePrefix + "_" + type][type][slug]), _c));
                                        }
                                    }
                                    return [2 /*return*/, data];
                            }
                        });
                    });
                } });
        }
        else {
            // slug -- object
            mixin = tslib_1.__assign({}, mixin, { asyncData: function (_a) {
                    var store = _a.store;
                    return tslib_1.__awaiter(this, void 0, void 0, function () {
                        var _b, _c, slug, dataName, meta, contentType, type;
                        return tslib_1.__generator(this, function (_d) {
                            switch (_d.label) {
                                case 0:
                                    _c = ConsumeObject(loaderRequest), slug = _c.slug, dataName = _c.dataName, meta = _c.meta, contentType = _c.contentType;
                                    type = ContentTypeToString(contentType);
                                    if (meta !== null) {
                                        metaSource = meta;
                                    }
                                    return [4 /*yield*/, store.dispatch(ModulePrefix + "_" + type + "/load", {
                                            slug: slug,
                                            type: contentType
                                        })];
                                case 1:
                                    _d.sent();
                                    return [2 /*return*/, (_b = {},
                                            _b[dataName] = orNull(store.state[ModulePrefix + "_" + type][type][slug]),
                                            _b)];
                            }
                        });
                    });
                } });
        }
    }
    // META
    mixin.mixins = [
        Meta('website', metaSource) // Page
    ];
    return mixin;
});
//# sourceMappingURL=loadSections.js.map